
use super::{JunctionState, UpstreamState};

#[test]
fn test_current_single() {
   let state = JunctionState {
       preference: 0,
       upstreams: vec![
           UpstreamState {
               address: "server".to_string(),
               is_up: true
           }
       ]
   };

    assert_eq!(state.current(), "server");
}

#[test]
fn test_current_replica() {
    let mut state = JunctionState {
        preference: 0,
        upstreams: vec![
            UpstreamState {
                address: "server-0".to_string(),
                is_up: true
            },
            UpstreamState {
                address: "server-1".to_string(),
                is_up: true
            }
        ]
    };

    /////////////////////////////////////////////////////
    state.preference = 0;

    // UP -- up
    state.upstreams[0].is_up = true;
    state.upstreams[1].is_up = true;
    assert_eq!(state.current(), "server-0");

    // DOWN -- up
    state.upstreams[0].is_up = false;
    state.upstreams[1].is_up = true;
    assert_eq!(state.current(), "server-1");

    // UP -- down
    state.upstreams[0].is_up = true;
    state.upstreams[1].is_up = false;
    assert_eq!(state.current(), "server-0");

    // DOWN -- down
    state.upstreams[0].is_up = false;
    state.upstreams[1].is_up = false;
    assert_eq!(state.current(), "server-0");

    /////////////////////////////////////////////////////
    state.preference = 1;

    // up -- UP
    state.upstreams[0].is_up = true;
    state.upstreams[1].is_up = true;
    assert_eq!(state.current(), "server-1");

    // down -- UP
    state.upstreams[0].is_up = false;
    state.upstreams[1].is_up = true;
    assert_eq!(state.current(), "server-1");

    // up -- DOWN
    state.upstreams[0].is_up = true;
    state.upstreams[1].is_up = false;
    assert_eq!(state.current(), "server-0");

    // down -- DOWN
    state.upstreams[0].is_up = false;
    state.upstreams[1].is_up = false;
    assert_eq!(state.current(), "server-1");
}

#[test]
fn test_advance() {
    let mut state = JunctionState {
        preference: 0,
        upstreams: vec![
            UpstreamState {
                address: "server-0".to_string(),
                is_up: true
            },
            UpstreamState {
                address: "server-1".to_string(),
                is_up: true
            }
        ]
    };

    assert_eq!(state.preference, 0);

    state.advance();
    assert_eq!(state.preference, 1);

    state.advance();
    assert_eq!(state.preference, 0);
}

#[test]
fn test_mark() {
    let mut state = JunctionState {
        preference: 0,
        upstreams: vec![
            UpstreamState {
                address: "server-0".to_string(),
                is_up: true
            },
            UpstreamState {
                address: "server-1".to_string(),
                is_up: true
            }
        ]
    };

    let changed = state.mark(0, false);
    assert_eq!(state.upstreams[0].is_up, false);
    assert_eq!(state.upstreams[1].is_up, true);
    assert_eq!(changed, true);

    let changed = state.mark(0, false);
    assert_eq!(changed, false);

    state.mark(1, false);
    assert_eq!(state.upstreams[0].is_up, false);
    assert_eq!(state.upstreams[1].is_up, false);

    state.mark(0, true);
    assert_eq!(state.upstreams[0].is_up, true);
    assert_eq!(state.upstreams[1].is_up, false);
}
